@@ -59,8 +59,8 @@ struct SyntaxDef     *TbarLayoutSyntaxPtr = &TbarLayoutSyntax;
 TermDef       MyFrameTerms[] = {
     {TF_NO_MYNAME_PREPENDING|TF_SYNTAX_START, "MyFrame", 7,     TT_QUOTED_TEXT, MYFRAME_START_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "North", 5,       TT_OPTIONAL_PATHNAME,    MYFRAME_North_ID, NULL},
-    {TF_NO_MYNAME_PREPENDING, "South", 5,       TT_OPTIONAL_PATHNAME,    MYFRAME_South_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "East", 4,        TT_OPTIONAL_PATHNAME,    MYFRAME_East_ID, NULL},
+    {TF_NO_MYNAME_PREPENDING, "South", 5,       TT_OPTIONAL_PATHNAME,    MYFRAME_South_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "West", 4,        TT_OPTIONAL_PATHNAME,    MYFRAME_West_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "NorthWest", 9,   TT_OPTIONAL_PATHNAME,    MYFRAME_NorthWest_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "NorthEast", 9,   TT_OPTIONAL_PATHNAME,    MYFRAME_NorthEast_ID, NULL},
@@ -83,23 +83,23 @@ TermDef       MyFrameTerms[] = {
     {TF_NO_MYNAME_PREPENDING, "FrameStickyStyle", 16,   TT_QUOTED_TEXT,  MYFRAME_FrameStickyStyle_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "FrameSStyle", 11,        TT_QUOTED_TEXT,  MYFRAME_FrameStickyStyle_ID, NULL},
     {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED, "SideSize", 8,        TT_GEOMETRY,MYFRAME_SideSize_ID, NULL},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideAlign", 9,       TT_FLAG,    MYFRAME_SideAlign_ID, &AlignSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideBevel", 9,       TT_FLAG,    MYFRAME_SideBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideFBevel", 10,       TT_FLAG,    MYFRAME_SideFBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideUBevel", 10,       TT_FLAG,    MYFRAME_SideUBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideSBevel", 10,       TT_FLAG,    MYFRAME_SideSBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideFocusedBevel", 16,      TT_FLAG,    MYFRAME_SideFBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideUnfocusedBevel", 18,    TT_FLAG,    MYFRAME_SideUBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideStickyBevel", 15,       TT_FLAG,    MYFRAME_SideSBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideAlign", 9,       TT_FLAG,    MYFRAME_SideAlign_ID, &AlignSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideBevel", 9,       TT_FLAG,    MYFRAME_SideBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideFBevel", 10,       TT_FLAG,    MYFRAME_SideFBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideUBevel", 10,       TT_FLAG,    MYFRAME_SideUBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideSBevel", 10,       TT_FLAG,    MYFRAME_SideSBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideFocusedBevel", 16,      TT_FLAG,    MYFRAME_SideFBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideUnfocusedBevel", 18,    TT_FLAG,    MYFRAME_SideUBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideStickyBevel", 15,       TT_FLAG,    MYFRAME_SideSBevel_ID, &BevelSyntax},
     {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED, "CornerSize", 10,     TT_GEOMETRY,MYFRAME_CornerSize_ID, NULL},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerAlign", 11,    TT_FLAG,    MYFRAME_CornerAlign_ID, &AlignSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerBevel", 11,    TT_FLAG,    MYFRAME_CornerBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerFBevel", 12,    TT_FLAG,    MYFRAME_CornerFBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerUBevel", 12,    TT_FLAG,    MYFRAME_CornerUBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerSBevel", 12,    TT_FLAG,    MYFRAME_CornerSBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerFocusedBevel", 18,   TT_FLAG,    MYFRAME_CornerFBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerUnfocusedBevel", 20, TT_FLAG,    MYFRAME_CornerUBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerStickyBevel", 17,    TT_FLAG,    MYFRAME_CornerSBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerAlign", 11,    TT_FLAG,    MYFRAME_CornerAlign_ID, &AlignSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerBevel", 11,    TT_FLAG,    MYFRAME_CornerBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerFBevel", 12,    TT_FLAG,    MYFRAME_CornerFBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerUBevel", 12,    TT_FLAG,    MYFRAME_CornerUBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerSBevel", 12,    TT_FLAG,    MYFRAME_CornerSBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerFocusedBevel", 18,   TT_FLAG,    MYFRAME_CornerFBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerUnfocusedBevel", 20, TT_FLAG,    MYFRAME_CornerUBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerStickyBevel", 17,    TT_FLAG,    MYFRAME_CornerSBevel_ID, &BevelSyntax},
     {TF_NO_MYNAME_PREPENDING, "TitleBevel", 10,         TT_FLAG,         MYFRAME_TitleBevel_ID, &BevelSyntax},
     {TF_NO_MYNAME_PREPENDING, "TitleFBevel", 11,         TT_FLAG,         MYFRAME_TitleFBevel_ID, &BevelSyntax},
     {TF_NO_MYNAME_PREPENDING, "TitleUBevel", 11,         TT_FLAG,         MYFRAME_TitleUBevel_ID, &BevelSyntax},
@@ -153,6 +153,14 @@ TermDef       MyFrameTerms[] = {
     {TF_NO_MYNAME_PREPENDING, "AllowBorder", 11,       		TT_FLAG,   MYFRAME_AllowBorder_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "LeftBtnAlign", 12,           TT_FLAG,   MYFRAME_LeftBtnAlign_ID, &AlignSyntax},
 	{TF_NO_MYNAME_PREPENDING, "RightBtnAlign", 13,          TT_FLAG,   MYFRAME_RightBtnAlign_ID, &AlignSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideSlicing", 11,       TT_GEOMETRY,    MYFRAME_SideSlicing_ID, NULL},
+    {TF_NO_MYNAME_PREPENDING, "LeftBtnBackSlicing", 18,       TT_GEOMETRY,     MYFRAME_LeftBtnBackSlicing_ID, NULL},
+    {TF_NO_MYNAME_PREPENDING, "LeftSpacerBackSlicing", 21,    TT_GEOMETRY,     MYFRAME_LeftSpacerBackSlicing_ID, NULL},
+    {TF_NO_MYNAME_PREPENDING, "LeftTitleSpacerBackSlicing", 26,    TT_GEOMETRY,     MYFRAME_LTitleSpacerBackSlicing_ID, NULL},
+	{TF_NO_MYNAME_PREPENDING, "TitleBackgroundSlicing", 22,   TT_GEOMETRY,         MYFRAME_TitleBackSlicing_ID, NULL},
+    {TF_NO_MYNAME_PREPENDING, "RightTitleSpacerBackSlicing", 27,   TT_GEOMETRY,     MYFRAME_RTitleSpacerBackSlicing_ID, NULL},
+    {TF_NO_MYNAME_PREPENDING, "RightSpacerBackSlicing", 22,   TT_GEOMETRY,   MYFRAME_RightSpacerBackSlicing_ID, NULL},
+    {TF_NO_MYNAME_PREPENDING, "RightBtnBackSlicing", 19,      TT_GEOMETRY,   MYFRAME_RightBtnBackSlicing_ID, NULL},
 
 	{0, NULL, 0, 0, 0}
 };
@@ -172,6 +180,13 @@ SyntaxDef     MyFrameSyntax = {
 	0
 };
 
+const char *frame_side_idx2name( int idx )
+{
+	if( idx >= 0 && idx < FRAME_PARTS )
+		return	MyFrameTerms[1+idx].keyword ;
+	return NULL;
+}	 
+
 /********************************************************************/
 /* First MyFrame processing code				    */
 /********************************************************************/
@@ -269,9 +284,24 @@ PrintMyFrameDefinitions (MyFrameDefinition * list, int index)
 		{
 	       fprintf (stderr, "MyFrame[%d].title_backs_align[%d] = 0x%lX;\n", index, i, list->title_backs_align[i]);
 		}
+    	for( i = 0 ; i < MYFRAME_TITLE_BACKS ; ++i )
+		{
+			if (list->title_backs_slicing[i].flags != 0)
+		       fprintf (stderr, "MyFrame[%d].title_backs_slicing[%d] = 0x%X, %dx%d%+d%+d;\n", 
+			            index, i, list->title_backs_slicing[i].flags,
+						list->title_backs_slicing[i].width, list->title_backs_slicing[i].height,
+						list->title_backs_slicing[i].x, list->title_backs_slicing[i].y);
+		}
         fprintf (stderr, "MyFrame[%d].title_fcm = %d;\n", index, list->title_fcm);
         fprintf (stderr, "MyFrame[%d].title_ucm = %d;\n", index, list->title_ucm);
         fprintf (stderr, "MyFrame[%d].title_scm = %d;\n", index, list->title_scm);
+		for (i = 0; i < FRAME_SIDES ; ++i)
+			if (list->side_slicing[i].flags != 0)
+			   fprintf (stderr, "MyFrame[%d].side_slicing[%d] = 0x%X, %dx%d%+d%+d;\n", 
+				        index, i, list->side_slicing[i].flags,
+						list->side_slicing[i].width, list->side_slicing[i].height,
+						list->side_slicing[i].x, list->side_slicing[i].y);
+
         if( list->inheritance_list )
             for( i = 0 ; i < list->inheritance_num ; ++i )
                 fprintf (stderr, "MyFrame[%d].Inherit[%d] = \"%s\";\n", index, i, list->inheritance_list[i]);
@@ -317,36 +347,12 @@ ParseTbarLayoutOptions( FreeStorageElem * options )
 void
 tbar_layout_parse(char *tline, FILE * fd, char **myname, int *playout)
 {
-    FilePtrAndData fpd ;
-    ConfigDef    *ConfigReader ;
-    FreeStorageElem *Storage = NULL, *more_stuff = NULL;
-	ConfigData cd ;
-    if( playout == NULL )
-        return;
-
-    fpd.fp = fd ;
-    fpd.data = safemalloc( strlen(tline)+1+1 ) ;
-    sprintf( fpd.data, "%s\n", tline );
-LOCAL_DEBUG_OUT( "fd(%p)->tline(\"%s\")->fpd.data(\"%s\")", fd, tline, fpd.data );
-	cd.fileptranddata = &fpd ;
-    ConfigReader = InitConfigReader ((char*)myname, &BevelSyntax, CDT_FilePtrAndData, cd, NULL);
-    free( fpd.data );
-
-    if (!ConfigReader)
-        return ;
-
-	PrintConfigReader (ConfigReader);
-	ParseConfig (ConfigReader, &Storage);
-	PrintFreeStorage (Storage);
-
-	/* getting rid of all the crap first */
-    StorageCleanUp (&Storage, &more_stuff, CF_DISABLED_OPTION);
-    DestroyFreeStorage (&more_stuff);
-
-    *playout = ParseTbarLayoutOptions(Storage);
-
-	DestroyConfig (ConfigReader);
-	DestroyFreeStorage (&Storage);
+    FreeStorageElem *Storage = tline_subsyntax_parse(NULL, tline, fd, (char*)myname, &BevelSyntax, NULL, NULL);
+	if( Storage )
+	{
+    	*playout = ParseTbarLayoutOptions(Storage);
+		DestroyFreeStorage (&Storage);
+	}
 }
 
 
@@ -389,6 +395,16 @@ ProcessMyFrameOptions (FreeStorageElem * options, MyFrameDefinition ** tail)
 						continue;
 					}
 				}
+				if( options->term->id >= MYFRAME_TitleBackSlicing_ID_START )
+				{
+					int index = options->term->id - MYFRAME_TitleBackSlicing_ID_START ;
+					if( index < MYFRAME_TITLE_BACKS )
+					{
+                        if (!ReadConfigItem (&item, options))
+                            continue;
+						fd->title_backs_slicing[index] = item.data.geometry;
+					}
+				}
                 switch( options->term->id )
                 {
                     case MYFRAME_TitleBevel_ID :
@@ -486,6 +502,9 @@ ProcessMyFrameOptions (FreeStorageElem * options, MyFrameDefinition ** tail)
                                 fd->part_sbevel[item.index] = ParseBevelOptions( options->sub );
                                 SetPartSBevelSet(fd,item.index );
                                 break;
+                            case MYFRAME_SideSlicing_ID :
+                                fd->side_slicing[item.index] = item.data.geometry;
+                                break;
                             default:
                                 show_warning( "Unexpected MyFrame definition keyword \"%s\" . Ignoring.", options->term->keyword );
                                 item.ok_to_free = 1;
@@ -690,39 +709,19 @@ MyFrameDefs2FreeStorage (SyntaxDef * syntax, FreeStorageElem ** tail, MyFrameDef
 void
 myframe_parse (char *tline, FILE * fd, char **myname, int *myframe_list)
 {
-    FilePtrAndData fpd ;
-    ConfigData cd ;
-	ConfigDef    *ConfigReader ;
-    FreeStorageElem *Storage = NULL, *more_stuff = NULL;
+    FreeStorageElem *Storage = NULL;
     MyFrameDefinition **list = (MyFrameDefinition**)myframe_list, **tail ;
 
     if( list == NULL )
         return;
     for( tail = list ; *tail != NULL ; tail = &((*tail)->next) );
 
-    fpd.fp = fd ;
-    fpd.data = safemalloc( 12+1+strlen(tline)+1+1 ) ;
-    sprintf( fpd.data, "MyFrame %s\n", tline );
-LOCAL_DEBUG_OUT( "fd(%p)->tline(\"%s\")->fpd.data(\"%s\")", fd, tline, fpd.data );
-    cd.fileptranddata = &fpd ;
-	ConfigReader = InitConfigReader ((char*)myname, &MyFrameSyntax, CDT_FilePtrAndData, cd, NULL);
-    free( fpd.data );
-
-    if (!ConfigReader)
-        return ;
-
-	PrintConfigReader (ConfigReader);
-	ParseConfig (ConfigReader, &Storage);
-	PrintFreeStorage (Storage);
-
-	/* getting rid of all the crap first */
-    StorageCleanUp (&Storage, &more_stuff, CF_DISABLED_OPTION);
-    DestroyFreeStorage (&more_stuff);
-
-    ProcessMyFrameOptions (Storage, tail);
-
-	DestroyConfig (ConfigReader);
-	DestroyFreeStorage (&Storage);
+    Storage = tline_subsyntax_parse("MyFrame", tline, fd, (char*)myname, &MyFrameSyntax, NULL, NULL);
+	if( Storage )
+	{
+    	ProcessMyFrameOptions (Storage, tail);
+		DestroyFreeStorage (&Storage);
+	}
 }
 
 
