@@ -28,7 +28,6 @@
 #include "afterconf.h"
 
 
-void          ProcessStatement (ConfigDef * config);
 /*****************************************************************************
  *
  * This routine is responsible for reading and parsing the base.<bpp> config
@@ -38,6 +37,7 @@ void          ProcessStatement (ConfigDef * config);
 
 #define WHARF_FOLDER_END 	"~Folder"
 
+
 TermDef       WhevTerms[] = {
 	{TF_NO_MYNAME_PREPENDING | TF_SYNTAX_TERMINATOR, "push", 4, TT_FILENAME, WHEV_PUSH_ID, NULL},
 	{TF_NO_MYNAME_PREPENDING | TF_SYNTAX_TERMINATOR, "close_folder", 12, TT_FILENAME, WHEV_CLOSE_FOLDER_ID, NULL},
@@ -77,10 +77,13 @@ SyntaxDef     WhevSyntax = {
     ASCF_DEFINE_KEYWORD(WHARF, 0, FlipLabel			, TT_FLAG		, NULL), \
     ASCF_DEFINE_KEYWORD(WHARF, 0, FitContents		, TT_FLAG		, NULL), \
     ASCF_DEFINE_KEYWORD(WHARF, 0, ShapeToContents	, TT_FLAG		, NULL), \
+    ASCF_DEFINE_KEYWORD(WHARF, 0, StretchBackground	, TT_FLAG		, NULL), \
     ASCF_DEFINE_KEYWORD(WHARF, 0, AlignContents		, TT_FLAG		, &AlignSyntax), \
     ASCF_DEFINE_KEYWORD(WHARF, 0, Bevel				, TT_FLAG		, &BevelSyntax), \
 	ASCF_DEFINE_KEYWORD(WHARF, 0, CompositionMethod	, TT_INTEGER	, NULL), \
-	ASCF_DEFINE_KEYWORD(WHARF, 0, FolderOffset		, TT_INTEGER	, NULL)
+	ASCF_DEFINE_KEYWORD(WHARF, 0, FolderOffset		, TT_INTEGER	, NULL), \
+	ASCF_DEFINE_KEYWORD(WHARF, 0, OrthogonalFolderOffset, TT_INTEGER	, NULL), \
+    ASCF_DEFINE_KEYWORD(WHARF, 0, ShowHints			, TT_FLAG		, &BalloonContentsSyntax)
 
 
 #define WHARF_PRIVATE_TERMS \
@@ -91,8 +94,8 @@ SyntaxDef     WhevSyntax = {
     {0, "ForceSize", 9,         TT_GEOMETRY, WHARF_ForceSize_ID, NULL}
 
 #define WHARF_FOLDER_TERMS \
-	{TF_SPECIAL_PROCESSING, "", 0, TT_SPECIAL, WHARF_Wharf_ID, &DummyFuncSyntax}, \
-	{TF_NO_MYNAME_PREPENDING|TF_SYNTAX_TERMINATOR, WHARF_FOLDER_END, 7, TT_FLAG, WHARF_FolderEnd_ID, NULL}
+	{TF_SPECIAL_PROCESSING|TF_NAMED|TF_USE_TOKENS(1), "", 0, TT_ICON_LIST, WHARF_Wharf_ID, &DummyFuncSyntax}, \
+	{TF_SYNTAX_TERMINATOR, WHARF_FOLDER_END, 7, TT_FLAG, WHARF_FolderEnd_ID, NULL}
 
 
 TermDef       WharfTerms[] = {
@@ -101,7 +104,6 @@ TermDef       WharfTerms[] = {
     {TF_OBSOLETE, "NoWithdraw", 10,       TT_FLAG, WHARF_NoWithdraw_ID, NULL},
 /* TextureType, MaxColors, BgColor, TextureColor, and Pixmap are obsolete */
 	{TF_OBSOLETE, "TextureType", 11, TT_UINTEGER, WHARF_TextureType_ID, NULL},
-    {TF_OBSOLETE, "MaxColors", 9,    TT_UINTEGER, WHARF_MaxColors_ID, NULL},
     {TF_OBSOLETE, "BgColor", 7,      TT_COLOR, WHARF_BgColor_ID, NULL},
     {TF_OBSOLETE, "TextureColor", 12,TT_COLOR, WHARF_TextureColor_ID, NULL},
     {TF_OBSOLETE, "Pixmap", 6,       TT_FILENAME, WHARF_Pixmap_ID, NULL},
@@ -154,7 +156,7 @@ TermDef       WharfFolderTerms[] = {
 SyntaxDef WharfFeelSyntax 		= {'\n', '\0', WharfFeelTerms, 		0, '\t', "", "\t", 	"WharfFeel", 	"WharfFeel", 	"AfterStep wharf module feel", NULL, 0};
 SyntaxDef WharfLookSyntax 		= {'\n', '\0', WharfLookTerms, 		0, '\t', "", "\t", 	"WharfLook", 	"WharfLook", 	"AfterStep wharf module look", NULL, 0};
 SyntaxDef WharfPrivateSyntax 	= {'\n', '\0', WharfPrivateTerms,	0, '\t', "", "\t", 	"Wharf",     	"Wharf",     	"AfterStep wharf module", NULL,0};
-SyntaxDef WharfFolderSyntax 	= {'\n', '\0', WharfFolderTerms,	0, '\t', "", "\t", 	"Wharf Folders","Wharf Folders","AfterStep wharf module", NULL,0};
+SyntaxDef WharfFolderSyntax 	= {'\n', '\0', WharfFolderTerms,	0, '\t', "", "\t", 	"Wharf Folders","WharfFolders", "AfterStep wharf module", NULL,0};
 SyntaxDef WharfSyntax 			= {'\n', '\0', WharfTerms,			0, ' ',  "", "",	"Module:Wharf",	"Wharf",	 	"AfterStep module for launching or docking applications aka button bar",NULL,0};
 
 
@@ -169,6 +171,7 @@ flag_options_xref WharfFlags[] = {
     {WHARF_FlipLabel, WHARF_FlipLabel_ID, 0},
     {WHARF_FitContents, WHARF_FitContents_ID, 0},
     {WHARF_ShapeToContents, WHARF_ShapeToContents_ID, 0 },
+    {WHARF_StretchBackground, WHARF_StretchBackground_ID, 0 },
     {0, 0, 0}
 };
 
@@ -183,7 +186,6 @@ CreateWharfButton ()
 void
 DestroyWharfButton (WharfButton **pbtn)
 {
-	register int  i;
     WharfButton *btn = *pbtn ;
 
 	if (btn == NULL)
@@ -193,6 +195,8 @@ DestroyWharfButton (WharfButton **pbtn)
     /* delete members */
 	if (btn->title != NULL)
 		free (btn->title);
+	if (btn->comment != NULL)
+		free (btn->comment);
 
 	if( btn->contents )
 	{
@@ -202,11 +206,7 @@ DestroyWharfButton (WharfButton **pbtn)
 			char **icon = btn->contents[k].icon ;
 			FunctionData *function = btn->contents[k].function ;
 			if (icon != NULL)
-			{
-				for (i = 0; icon[i] != NULL; i++)
-					free (icon[i]);
-				free (icon);
-			}
+				destroy_string_list( icon, 0 );
 
 			if (function)
 			{
@@ -236,11 +236,13 @@ CreateWharfConfig ()
     config->withdraw_style = WITHDRAW_ON_EDGE_BUTTON_AND_SHOW ;
     
 	config->AlignContents = WHARF_DEFAULT_AlignContents ;
+	config->ShowHints = WHARF_DEFAULT_ShowHints ;
 
 	config->more_stuff = NULL;
 
     config->CompositionMethod = WHARF_DEFAULT_CompositionMethod ;
 	config->FolderOffset = WHARF_DEFAULT_FolderOffset ;
+	config->OrthogonalFolderOffset = WHARF_DEFAULT_OrthogonalFolderOffset ;
 
 	return config;
 }
@@ -348,10 +350,13 @@ PrintWharfConfig(WharfConfig *config )
 	ASCF_PRINT_FLAG_KEYWORD(stderr,WHARF,config,FlipLabel);
 	ASCF_PRINT_FLAG_KEYWORD(stderr,WHARF,config,FitContents);
 	ASCF_PRINT_FLAG_KEYWORD(stderr,WHARF,config,ShapeToContents );
+	ASCF_PRINT_FLAG_KEYWORD(stderr,WHARF,config,StretchBackground );
 	ASCF_PRINT_FLAGS_KEYWORD(stderr,WHARF,config,AlignContents );
 	ASCF_PRINT_FLAGS_KEYWORD(stderr,WHARF,config,Bevel );
+	ASCF_PRINT_FLAGS_KEYWORD(stderr,WHARF,config,ShowHints );
 	ASCF_PRINT_INT_KEYWORD(stderr,WHARF,config,CompositionMethod);
 	ASCF_PRINT_INT_KEYWORD(stderr,WHARF,config,FolderOffset);
+	ASCF_PRINT_INT_KEYWORD(stderr,WHARF,config,OrthogonalFolderOffset);
     
 	print_wharf_folder( config->root_folder, 1 );
 
@@ -360,13 +365,13 @@ PrintWharfConfig(WharfConfig *config )
 void print_trimmed_str( char *prompt, char * str );
 
 unsigned long
-WharfSpecialFunc (ConfigDef * config, FreeStorageElem ** storage)
+WharfSpecialFunc (ConfigDef * config)
 {
 	TermDef      *pterm;
 	register char *cur;
 
-    LOCAL_DEBUG_CALLER_OUT("%p,%p", config, storage);
-	if (config == NULL || storage == NULL)
+    LOCAL_DEBUG_CALLER_OUT("%p", config);
+	if (config == NULL )
 		return SPECIAL_BREAK;
 
 	/* checking if we have ~Folders in here */
@@ -413,7 +418,7 @@ WharfSpecialFunc (ConfigDef * config, FreeStorageElem ** storage)
 		config->cursor = cur;
         /* we are at the beginning of the function definition right now - lets process it :*/
         /* read in entire function definition */
-		GetNextStatement (config, 1);
+		GetNextStatement (config);
         /* lets find us the term for this definition */
         print_trimmed_str( "config->current_data", config->current_data );
         LOCAL_DEBUG_OUT( "curr_data_len = %d", config->current_data_len);
@@ -523,9 +528,7 @@ ParseWharfItem (FreeStorageElem * storage, WharfButton **folder)
 
             if (null_icon > 0)
             {
-                if (new_icon_list[0] != NULL)
-                    free (new_icon_list[0]);
-                free (new_icon_list);
+				destroy_string_list( new_icon_list, 0);		
                 new_icon_list = NULL;
             }
         }
@@ -614,11 +617,10 @@ ParseWharfOptions (const char *filename, char *myname)
 	WharfConfig  *config = CreateWharfConfig ();
 	FreeStorageElem *Storage = NULL, *pCurr;
 	ConfigItem    item;
-	MyStyleDefinition **styles_tail = &(config->style_defs);
 	TermDef      *folder_term = func2fterm (F_Folder, False);
 
 	cd.filename = filename ;
-	ConfigReader = InitConfigReader (myname, &WharfSyntax, CDT_Filename, cd, WharfSpecialFunc);
+	ConfigReader = InitConfigReader (myname, &WharfSyntax, CDT_Filename, cd, NULL /*WharfSpecialFunc*/);
 	if (!ConfigReader)
 		return config;
 
@@ -626,15 +628,12 @@ ParseWharfOptions (const char *filename, char *myname)
 
     item.memory = NULL;
 	PrintConfigReader (ConfigReader);
-SHOW_CHECKPOINT;
 	ParseConfig (ConfigReader, &Storage);
-SHOW_CHECKPOINT;
-	PrintFreeStorage (Storage);
 
 	/* getting rid of all the crap first */
 	StorageCleanUp (&Storage, &(config->more_stuff), CF_DISABLED_OPTION);
-SHOW_CHECKPOINT;
-	config->balloon_conf = Process_balloonOptions (Storage, NULL);
+	config->balloon_conf = Process_balloonOptions (Storage, NULL, BALLOON_ID_START);
+	config->style_defs = free_storage2MyStyleDefinitionsList (Storage);
 
 	for (pCurr = Storage; pCurr; pCurr = pCurr->next)
 	{
@@ -683,10 +682,6 @@ SHOW_CHECKPOINT;
 			 set_flags (config->set_flags, WHARF_TEXTURE_TYPE);
 			 config->texture_type = item.data.integer;
 			 break;
-		 case WHARF_MaxColors_ID:
-			 set_flags (config->set_flags, WHARF_MAX_COLORS);
-			 config->max_colors = item.data.integer;
-			 break;
 		 case WHARF_BgColor_ID:
 			 set_string_value (&(config->bg_color), item.data.string, &(config->set_flags), WHARF_BG_COLOR);
 			 break;
@@ -729,13 +724,10 @@ SHOW_CHECKPOINT;
 		ASCF_HANDLE_INTEGER_KEYWORD_CASE(WHARF,config,item,LabelLocation ); 
 		ASCF_HANDLE_ALIGN_KEYWORD_CASE(WHARF,config,pCurr,AlignContents ); 
 		ASCF_HANDLE_BEVEL_KEYWORD_CASE(WHARF,config,pCurr,Bevel); 
+		ASCF_HANDLE_SUBSYNTAX_KEYWORD_CASE(WHARF,config,pCurr,ShowHints,BalloonContents);
 		ASCF_HANDLE_INTEGER_KEYWORD_CASE(WHARF,config,item,CompositionMethod ); 
 		ASCF_HANDLE_INTEGER_KEYWORD_CASE(WHARF,config,item,FolderOffset ); 
-         case MYSTYLE_START_ID:
-			 styles_tail = ProcessMyStyleOptions (pCurr->sub, styles_tail);
-			 item.ok_to_free = 1;
-			 break;
-
+		ASCF_HANDLE_INTEGER_KEYWORD_CASE(WHARF,config,item,OrthogonalFolderOffset ); 
 		 default:
 			 if (pCurr->term->type != TT_FLAG)
 				 item.ok_to_free = 1;
@@ -754,6 +746,7 @@ SHOW_CHECKPOINT;
     return config;
 }
 
+
 #if 0
 
 FreeStorageElem **WharfFolder2FreeStorage (SyntaxDef * syntax, FreeStorageElem ** tail, WharfButton * folder, int root);
@@ -859,10 +852,12 @@ WriteWharfOptions (const char *filename, char *myname, WharfConfig * config, uns
 
     CopyFreeStorage (&Storage, config->more_stuff);
 
+    if (config->style_defs)
+		*tail = MyStyleDefinitionsList2free_storage (config->style_defs, &WharfSyntax);
+
 	if (config->balloon_conf)
 		tail = balloon2FreeStorage (&WharfSyntax, tail, config->balloon_conf);
-    if (config->style_defs)
-        tail = MyStyleDefs2FreeStorage (&WharfSyntax, tail, config->style_defs);
+
 	/* building free storage here */
 	/* geometry */
 	if (get_flags (config->set_flags, WHARF_GEOMETRY))
@@ -882,9 +877,6 @@ WriteWharfOptions (const char *filename, char *myname, WharfConfig * config, uns
 	/* texture_type  */
 	if (get_flags (config->set_flags, WHARF_TEXTURE_TYPE))
         tail = Integer2FreeStorage (&WharfSyntax, tail, NULL, config->texture_type, WHARF_TextureType_ID);
-	/* max_colors */
-	if (get_flags (config->set_flags, WHARF_MAX_COLORS))
-        tail = Integer2FreeStorage (&WharfSyntax, tail, NULL, config->max_colors, WHARF_MaxColors_ID);
 	/* animate_steps */
 	if (get_flags (config->set_flags, WHARF_ANIMATE_STEPS))
         tail = Integer2FreeStorage (&WharfSyntax, tail, NULL, config->animate_steps, WHARF_AnimateSteps_ID);
@@ -922,7 +914,8 @@ WriteWharfOptions (const char *filename, char *myname, WharfConfig * config, uns
 		tail = WharfFolder2FreeStorage (&WharfSyntax, tail, config->root_folder, True);
 
 	/* writing config into the file */
-	WriteConfig (WharfConfigWriter, &Storage, CDT_Filename, (void **)&filename, flags);
+	WriteConfig (WharfConfigWriter, Storage, CDT_Filename, (void **)&filename, flags);
+	DestroyFreeStorage (&Storage);
 	DestroyConfig (WharfConfigWriter);
 
 	if (Storage)
