@@ -34,9 +34,15 @@
 # endif
 #else
 #include <setjmp.h>
+# ifdef HAVE_JPEG
+#   ifdef HAVE_UNISTD_H
+#     include <unistd.h>
+#   endif
+#   include <stdio.h>
+# endif
 #endif
 #ifdef HAVE_JPEG
-/* Include file for users of png library. */
+/* Include file for users of jpg library. */
 # undef HAVE_STDLIB_H
 # ifndef X_DISPLAY_MISSING
 #  include <X11/Xmd.h>
@@ -267,17 +273,21 @@ ASImage2xpm ( ASImage *im, const char *path, ASImageExportParams *params )
 	FILE *outfile;
 	unsigned int y, x ;
 	int *mapped_im, *row_pointer ;
-	ASColormap         cmap;
-	ASXpmCharmap       xpm_cmap ;
+	ASColormap         cmap = {0};
+	ASXpmCharmap       xpm_cmap = {0};
 	int transp_idx = 0;
 	START_TIME(started);
-	ASXpmExportParams defaults = { ASIT_Xpm, EXPORT_ALPHA, 4, 127, 512 } ;
+	static const ASXpmExportParams defaultsXPM = { ASIT_Xpm, EXPORT_ALPHA, 4, 127, 512 };
+	ASImageExportParams defaults;
 	register char *ptr ;
 
 	LOCAL_DEBUG_CALLER_OUT ("(\"%s\")", path);
 
-	if( params == NULL )
-		params = (ASImageExportParams *)&defaults ;
+	if( params == NULL ) {
+           defaults.type = defaultsXPM.type;
+           defaults.xpm = defaultsXPM;
+           params = &defaults ;
+        }
 
 	if ((outfile = open_writeable_image_file( path )) == NULL)
 		return False;
@@ -326,7 +336,8 @@ LOCAL_DEBUG_OUT( "(%d,%d)->%d (row_pointer %d )", x, y, idx, row_pointer[x] );
 		fputc( '\n', outfile );
 	}
 	fprintf( outfile, "};\n" );
-	fclose( outfile );
+	if (outfile != stdout)
+		fclose( outfile );
 
 	SHOW_TIME("image writing",started);
 	destroy_xpm_charmap( &xpm_cmap, True );
@@ -337,6 +348,120 @@ LOCAL_DEBUG_OUT( "(%d,%d)->%d (row_pointer %d )", x, y, idx, row_pointer[x] );
 	return True;
 }
 
+
+/****** VO ******/
+Bool
+ASImage2xpmRawBuff ( ASImage *im, CARD8 **buffer, int *size, ASImageExportParams *params )
+{
+	unsigned int y, x ;
+	int *mapped_im, *row_pointer ;
+	ASColormap         cmap = {0};
+	ASXpmCharmap       xpm_cmap = {0} ;
+	int transp_idx = 0;
+	START_TIME(started);
+	static const ASXpmExportParams defaultsXPM = { ASIT_Xpm, EXPORT_ALPHA, 4, 127, 512 };
+        ASImageExportParams defaults;
+	register char *ptr ;
+   char *curr;
+
+   if( params == NULL ) {
+      defaults.type = defaultsXPM.type;
+      defaults.xpm = defaultsXPM;
+      params = &defaults ;
+   }
+
+    mapped_im = colormap_asimage( im, &cmap, params->xpm.max_colors, params->xpm.dither, params->xpm.opaque_threshold );
+	if (mapped_im == NULL)
+		return False;
+	if( !get_flags( params->xpm.flags, EXPORT_ALPHA) )
+		cmap.has_opaque = False ;
+	else
+		transp_idx = cmap.count ;
+
+
+LOCAL_DEBUG_OUT("building charmap%s","");
+	build_xpm_charmap( &cmap, cmap.has_opaque, &xpm_cmap );
+	SHOW_TIME("charmap calculation",started);
+
+   *size = 0;
+   *buffer = 0;
+
+   /* crazy check against buffer overflow */
+   if ((im->width > 100000) || (im->height > 1000000) || 
+       (xpm_cmap.count > 100000) || (xpm_cmap.cpp > 100000)) {
+		destroy_xpm_charmap( &xpm_cmap, True );
+		free( mapped_im );
+		destroy_colormap( &cmap, True );
+      return False;
+   }
+
+   /* estimate size*/
+   *size =  (im->width + 4)*im->height*xpm_cmap.cpp;
+   *size += cmap.count*(20 + xpm_cmap.cpp);
+   *size += 200;
+
+   curr = calloc(*size, 1);
+   *buffer = (CARD8*)curr;
+
+	sprintf(curr, "/* XPM */\nstatic char *asxpm[] = {\n/* columns rows colors chars-per-pixel */\n"
+					  "\"%d %d %d %d\",\n", im->width, im->height, xpm_cmap.count,  xpm_cmap.cpp );
+
+   curr += strlen(curr);
+
+    ptr = &(xpm_cmap.char_code[0]);
+	for( y = 0 ; y < cmap.count ; y++ )
+	{
+		sprintf(curr, "\"%s c #%2.2X%2.2X%2.2X\",\n", ptr, cmap.entries[y].red, cmap.entries[y].green, cmap.entries[y].blue );
+		ptr += xpm_cmap.cpp+1 ;
+      curr += strlen(curr);
+	}
+	if( cmap.has_opaque && y < xpm_cmap.count ) {
+		sprintf(curr, "\"%s c None\",\n", ptr );
+      curr += strlen(curr);
+   }
+	SHOW_TIME("image header writing",started);
+
+	row_pointer = mapped_im ;
+	for( y = 0 ; y < im->height ; y++ )
+	{
+		*curr = '"';
+      curr++;
+
+		for( x = 0; x < im->width ; x++ )
+		{
+			register int idx = (row_pointer[x] >= 0)? row_pointer[x] : transp_idx ;
+			register char *ptr = &(xpm_cmap.char_code[idx*(xpm_cmap.cpp+1)]) ;
+         int len = strlen(ptr);
+
+            if( idx > (int)cmap.count )
+                show_error("bad XPM color index :(%d,%d) -> %d, %d: %s", x, y, idx, row_pointer[x], ptr );
+
+         memcpy(curr, ptr, len);
+         curr += len;
+		}
+		row_pointer += im->width ;
+		*curr = '"';
+      curr++;
+		if( y < im->height-1 ) {
+		   *curr = ',';
+         curr++;
+      }
+		*curr = '\n';
+      curr++;
+	}
+	sprintf( curr, "};\n" );
+
+	destroy_xpm_charmap( &xpm_cmap, True );
+	free( mapped_im );
+	destroy_colormap( &cmap, True );
+   *size = strlen((char*)*buffer);
+
+	SHOW_TIME("total",started);
+	return True;
+}
+
+
+
 #else  			/* XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM */
 
 Bool
@@ -366,7 +491,7 @@ ASImage2png_int ( ASImage *im, void *data, png_rw_ptr write_fn, png_flush_ptr fl
 	png_color_16 back_color ;
 
 	START_TIME(started);
-	static ASPngExportParams defaults = { ASIT_Png, EXPORT_ALPHA, -1 };
+	static const ASPngExportParams defaults = { ASIT_Png, EXPORT_ALPHA, -1 };
 
 	png_ptr = png_create_write_struct( PNG_LIBPNG_VER_STRING, NULL, NULL, NULL );
     if ( png_ptr != NULL )
@@ -527,7 +652,8 @@ ASImage2png ( ASImage *im, const char *path, register ASImageExportParams *param
 
 	res = ASImage2png_int ( im, outfile, NULL, NULL, params );
 	
-	fclose(outfile);
+	if (outfile != stdout)
+		fclose(outfile);
 	return res;
 }
 
@@ -569,7 +695,7 @@ ASImage2PNGBuff( ASImage *im, CARD8 **buffer, int *size, ASImageExportParams *pa
 	
 	memset( &int_buff, 0x00, sizeof(ASImPNGBuffer) );
 
- 	if( ASImage2png_int ( im, &int_buff, asim_png_write_data, asim_png_flush_data, params ) )
+ 	if( ASImage2png_int ( im, &int_buff, (png_rw_ptr)asim_png_write_data, (png_flush_ptr)asim_png_flush_data, params ) )
 	{
 		*buffer	= int_buff.buffer ; 
 		*size = int_buff.used_size ; 		   
@@ -622,7 +748,8 @@ ASImage2jpeg( ASImage *im, const char *path,  ASImageExportParams *params )
 	FILE 		 *outfile;		/* target file */
     JSAMPROW      row_pointer[1];/* pointer to JSAMPLE row[s] */
 	int 		  y;
-	ASJpegExportParams defaults = { ASIT_Jpeg, 0, -1 };
+	static const ASJpegExportParams defaultsJpeg = { ASIT_Jpeg, 0, -1 };
+	ASImageExportParams defaults;
 	Bool grayscale;
 	ASImageDecoder *imdec ;
 	CARD32 *r, *g, *b ;
@@ -631,8 +758,11 @@ ASImage2jpeg( ASImage *im, const char *path,  ASImageExportParams *params )
 	if( im == NULL )
 		return False;
 
-	if( params == NULL )
-		params = (ASImageExportParams *)&defaults ;
+	if( params == NULL ) {
+           defaults.type = defaultsJpeg.type;
+           defaults.jpeg = defaultsJpeg;
+           params = &defaults ;
+        }
 
 	if ((outfile = open_writeable_image_file( path )) == NULL)
 		return False;
@@ -642,7 +772,8 @@ ASImage2jpeg( ASImage *im, const char *path,  ASImageExportParams *params )
 									  0, 0, im->width, 0, NULL)) == NULL )
 	{
 		LOCAL_DEBUG_OUT( "failed to start image decoding%s", "");
-		fclose(outfile);
+		if (outfile != stdout)
+			fclose(outfile);
 		return False;
 	}
 
@@ -738,7 +869,8 @@ LOCAL_DEBUG_OUT( "done writing image%s","" );
 	free( row_pointer[0] );
 	
 	stop_image_decoding( &imdec );
-	fclose(outfile);
+	if (outfile != stdout)
+		fclose(outfile);
 
 	SHOW_TIME("image export",started);
 	LOCAL_DEBUG_OUT("done writing JPEG image \"%s\"", path);
@@ -843,46 +975,48 @@ bmp_write16 (FILE *fp, CARD16 *data, int count)
 Bool
 ASImage2bmp ( ASImage *im, const char *path,  ASImageExportParams *params )
 {
+	Bool success = False;
 	FILE *outfile = NULL ;
-	BITMAPINFO *bmi ;
-	void *bmbits ;
-	int bits_size ;
-	BITMAPFILEHEADER bmh ;
 	START_TIME(started);
 
-	if ((outfile = open_writeable_image_file( path )) == NULL)
-		return False;
+	if ((outfile = open_writeable_image_file( path )) != NULL)
+	{
+		void *bmbits ;
+		BITMAPINFO *bmi = ASImage2DBI( get_default_asvisual(), im, 0, 0, im->width, im->height, &bmbits, 0 );
+		if( bmi != NULL && bmbits != NULL ) 
+		{
+			BITMAPFILEHEADER bmh ;
+			int bits_size = (((bmi->bmiHeader.biWidth*3+3)/4)*4)*bmi->bmiHeader.biHeight;          /* DWORD aligned */
 
-	bmi = ASImage2DBI( get_default_asvisual(), im, 0, 0, im->width, im->height, &bmbits );
-	if( bmi == NULL || bmbits == NULL ) 
-		return False ;
- 
-	bits_size = (((bmi->bmiHeader.biWidth*3+3)/4)*4)*bmi->bmiHeader.biHeight;          /* DWORD aligned */
+			bmh.bfType = BMP_SIGNATURE;
+		    bmh.bfSize = 14+bmi->bmiHeader.biSize+bits_size; /* Specifies the size, in bytes, of the bitmap file */
+		    bmh.bfReserved1 = 0;
+			bmh.bfReserved2 = 0;
+		    bmh.bfOffBits = 14+bmi->bmiHeader.biSize; /* Specifies the offset, in bytes, 
+							   * from the BITMAPFILEHEADER structure to the bitmap bits */
+			/* writing off the header */
+			bmp_write16( outfile, &bmh.bfType, 1 );
+			bmp_write32( outfile, &bmh.bfSize, 3 );
+			/* writing off the bitmapinfo : */
+			bmp_write32( outfile, &bmi->bmiHeader.biSize, 1 );
+			bmp_write32( outfile, (CARD32*)&bmi->bmiHeader.biWidth, 2 );
+			bmp_write16( outfile, &bmi->bmiHeader.biPlanes, 2 );
+			/* bmi->bmiHeader.biCompression = 0 ; */
+			bmp_write32( outfile, &bmi->bmiHeader.biCompression, 6 );
 
-	bmh.bfType = BMP_SIGNATURE;
-    bmh.bfSize = 14+bmi->bmiHeader.biSize+bits_size; /* Specifies the size, in bytes, of the bitmap file */
-    bmh.bfReserved1 = 0;
-	bmh.bfReserved2 = 0;
-    bmh.bfOffBits = 14+bmi->bmiHeader.biSize; /* Specifies the offset, in bytes, 
-					   * from the BITMAPFILEHEADER structure to the bitmap bits */
-	/* writing off the header */
-	bmp_write16( outfile, &bmh.bfType, 1 );
-	bmp_write32( outfile, &bmh.bfSize, 3 );
-	/* writing off the bitmapinfo : */
-	bmp_write32( outfile, &bmi->bmiHeader.biSize, 1 );
-	bmp_write32( outfile, (CARD32*)&bmi->bmiHeader.biWidth, 2 );
-	bmp_write16( outfile, &bmi->bmiHeader.biPlanes, 2 );
-	bmi->bmiHeader.biCompression = 1 ;
-	bmp_write32( outfile, &bmi->bmiHeader.biCompression, 6 );
-
-	/* writing off the bitmapbits */
-	fwrite( bmbits, sizeof(CARD8), bits_size, outfile );
-	free( bmbits );
-	free( bmi );
-
-	fclose(outfile);
+			/* writing off the bitmapbits */
+			if (fwrite( bmbits, sizeof(CARD8), bits_size, outfile ) == bits_size)
+				success = True;
+				
+			free( bmbits );
+			free( bmi );
+			
+		}
+		if (outfile != stdout)
+			fclose(outfile);
+	}
 	SHOW_TIME("image export",started);
-	return False;
+	return success;
 }
 
 /***********************************************************************************/
@@ -905,7 +1039,8 @@ Bool ASImage2gif( ASImage *im, const char *path,  ASImageExportParams *params )
 	GifFileType *gif = NULL ;
 	ColorMapObject *gif_cmap ;
 	Bool dont_save_cmap = False ;
-	ASGifExportParams defaults = { ASIT_Gif,EXPORT_ALPHA|EXPORT_APPEND, 3, 127, 10 };
+	static const ASGifExportParams defaultsGif = { ASIT_Gif,EXPORT_ALPHA|EXPORT_APPEND, 3, 127, 10 };
+        ASImageExportParams defaults;
 	ASColormap         cmap;
 	int *mapped_im ;
 	int y ;
@@ -923,8 +1058,11 @@ Bool ASImage2gif( ASImage *im, const char *path,  ASImageExportParams *params )
 															   		*/
 	LOCAL_DEBUG_CALLER_OUT ("(\"%s\")", path);
 
-	if( params == NULL )
-		params = (ASImageExportParams *)&defaults ;
+	if( params == NULL ) {
+           defaults.type = defaultsGif.type;
+           defaults.gif = defaultsGif;
+           params = &defaults ;
+        }
 
 	mapped_im = colormap_asimage( im, &cmap, 255, params->gif.dither, params->gif.opaque_threshold );
 
@@ -950,6 +1088,7 @@ Bool ASImage2gif( ASImage *im, const char *path,  ASImageExportParams *params )
 		cmap_size = cmap_size<<1 ;
 	if( (gif_cmap = MakeMapObject(cmap_size, NULL )) == NULL )
 	{
+		free( mapped_im );
 		ASIM_PrintGifError();
 		return False;
 	}
@@ -967,9 +1106,15 @@ Bool ASImage2gif( ASImage *im, const char *path,  ASImageExportParams *params )
 		{
 			ASIM_PrintGifError();
 			if( gif )
+			{
 				DGifCloseFile(gif);
-			fclose( infile );
-			gif = NULL ;
+				gif = NULL ;
+			}
+			if (infile)
+			{
+				fclose( infile );
+				infile = NULL;
+			}
 		}else
 		{
 			GifFileType gif_src ;
@@ -979,10 +1124,15 @@ Bool ASImage2gif( ASImage *im, const char *path,  ASImageExportParams *params )
 			gif->SColorMap = NULL ;
 			gif->Image.ColorMap = NULL ;
 			DGifCloseFile(gif);
-			fclose( infile );
+			gif = NULL;
+			fclose (infile);
+			infile = NULL;
 			outfile = open_writeable_image_file( path );
-			gif = EGifOpenFileHandle(fileno(outfile));
-			if( gif != NULL )
+
+			if (outfile)
+				gif = EGifOpenFileHandle(fileno(outfile));
+				
+			if (gif)
 			{
 				int status;
 				if( ( status = EGifPutScreenDesc(gif, gif_src.SWidth, gif_src.SHeight,
@@ -993,14 +1143,14 @@ Bool ASImage2gif( ASImage *im, const char *path,  ASImageExportParams *params )
 				if( status != GIF_OK )
 					ASIM_PrintGifError();
 			}
-			if( gif_src.SColorMap )
+			if (gif_src.SColorMap)
 			{  /* we only want to save private colormap if it is any different from
 			    * screen colormap ( saves us  768 bytes per image ) */
 				if( gif_cmap->ColorCount == gif_src.SColorMap->ColorCount )
 					dont_save_cmap = ( memcmp( gif_cmap->Colors, gif_src.SColorMap->Colors, gif_cmap->ColorCount*sizeof(GifColorType)) == 0 );
 				FreeMapObject(gif_src.SColorMap);
 			}
-			if( gif )
+			if (gif)
 			{
 				EGifPutExtension(gif, GRAPHICS_EXT_FUNC_CODE, GIF_GCE_BYTES, &(gce_bytes[0]));
 				if( get_flags( params->gif.flags, EXPORT_ANIMATION_REPEATS ) )
@@ -1016,18 +1166,23 @@ Bool ASImage2gif( ASImage *im, const char *path,  ASImageExportParams *params )
 		free_gif_saved_images( images, count );
 	}
 
-	if( gif == NULL )
+	if (gif == NULL)
 	{
-		outfile = open_writeable_image_file( path );
-		if( (gif = EGifOpenFileHandle(fileno(outfile))) == NULL )
-			ASIM_PrintGifError();
+		if (outfile == NULL)
+			outfile = open_writeable_image_file(path);
+			
+		if (outfile)
+			if ((gif = EGifOpenFileHandle(fileno(outfile))) == NULL)
+				ASIM_PrintGifError();
 	}
 
 	if( new_image && gif )
 	{
 		if( EGifPutScreenDesc(gif, im->width, im->height, cmap_size, 0, gif_cmap ) == GIF_ERROR )
 			ASIM_PrintGifError();
+	
 		EGifPutExtension(gif, 0xf9, GIF_GCE_BYTES, &(gce_bytes[0]));
+	
 		if( EGifPutImageDesc(gif, 0, 0, im->width, im->height, FALSE, NULL ) == GIF_ERROR )
 			ASIM_PrintGifError();
 	}
@@ -1054,9 +1209,19 @@ Bool ASImage2gif( ASImage *im, const char *path,  ASImageExportParams *params )
 		free( row_pointer );
 		if (EGifCloseFile(gif) == GIF_ERROR)
 			ASIM_PrintGifError();
+		gif = NULL;
 	}
 	free( mapped_im );
 	destroy_colormap( &cmap, True );
+	
+	if (outfile 
+#ifdef NO_DOUBLE_FCLOSE_AFTER_FDOPEN
+		&& gif  /* can't do double fclose in MS CRT after VC2005 */
+#endif
+		&& outfile != stdout)
+	{
+		fclose (outfile);
+	}
 
 	SHOW_TIME("image export",started);
 	return True ;
@@ -1075,7 +1240,8 @@ Bool
 ASImage2tiff( ASImage *im, const char *path, ASImageExportParams *params)
 {
 	TIFF *out;
-	ASTiffExportParams defaults = { ASIT_Tiff, 0, -1, TIFF_COMPRESSION_NONE, 100 };
+	static const ASTiffExportParams defaultsTiff = { ASIT_Tiff, 0, -1, TIFF_COMPRESSION_NONE, 100, 0 };
+        ASImageExportParams defaults;
 	uint16 photometric = PHOTOMETRIC_RGB;
 	tsize_t linebytes, scanline;
 	ASImageDecoder *imdec ;
@@ -1086,8 +1252,11 @@ ASImage2tiff( ASImage *im, const char *path, ASImageExportParams *params)
 	int nsamples = 3 ;
 	START_TIME(started);
 
-	if( params == NULL )
-		params = (ASImageExportParams *)&defaults ;
+	if( params == NULL ) {
+           defaults.type = defaultsTiff.type;
+           defaults.tiff = defaultsTiff;
+           params = &defaults ;
+        }
 
 	if( path == NULL )
 	{
@@ -1133,7 +1302,7 @@ ASImage2tiff( ASImage *im, const char *path, ASImageExportParams *params)
 	TIFFSetField(out, TIFFTAG_BITSPERSAMPLE,   8);
 	TIFFSetField(out, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
 	if( params->tiff.compression_type == -1  )
-		params->tiff.compression_type = defaults.compression_type ;
+		params->tiff.compression_type = defaultsTiff.compression_type ;
 	TIFFSetField(out, TIFFTAG_COMPRESSION,  params->tiff.compression_type);
 	switch (params->tiff.compression_type )
 	{
